WITH
req AS  (SELECT $$app_req$$::jsonb j)
,target AS (
    SELECT
        (req.j->>'app_vincr')::numeric vincr   --volume
        ,(req.j->>'app_pincr')::numeric pincr  --price
    FROM
        req
)
-----this block is supposed to test for new products that might not be in baseline etc-------
,test AS (
    SELECT
        sum(app_units) FILTER (WHERE version <> 'ACTUALS') total
        ,sum(app_units) FILTER (WHERE iter = 'baseline') base
    FROM
        tpsv.dcard o 
    WHERE
        app_where
)
,basemix AS (
SELECT
    o.id
    ,o.logid
    ,o."Trans. Date"
    ,o."Post Date"
    ,o."Description"
    ,o."Amount"
    ,o."Category"
    ,o."Party"
    ,o."Reason"
    ,o.app_units
FROM
    tpsv.dcard o 
WHERE
    app_where
),
vscale AS (
    SELECT
        (SELECT vincr FROM target) AS target_increment
        ,sum(app_units) AS units
        ,CASE WHEN sum(app_units) = 0 THEN 0 ELSE (SELECT vincr FROM target)/sum(app_units) END AS factor
    FROM
        basemix
)
,ilog AS (
    INSERT INTO
        fc.log (doc)
    SELECT
        req.j 
    FROM
        req
)
,volume AS (
SELECT
    o.id
    ,o.logid
    ,o."Trans. Date"
    ,o."Post Date"
    ,o."Description"
    ,o."Amount" * vscale.factor AS "Amount"
    ,o."Category"
    ,o."Party"
    ,o."Reason"
    ,o.app_units * vscale.factor AS app_units
    ,'app_forecast_name' AS version
    ,'scale vol' AS iter
FROM
    basemix o
    CROSS JOIN vscale
)
,pscale AS (
SELECT
    (SELECT pincr FROM target) AS target_increment
    ,sum("Amount") AS value
    ,CASE WHEN (SELECT sum("Amount") FROM volume) = 0 THEN
        --if the base value is -0- scaling will not work, need to generate price, factor goes to -0-
        0
    ELSE
        --if the target dollar value still does not match the target increment, make this adjustment
        ((SELECT pincr FROM target)-(SELECT sum("Amount") FROM volume))/(SELECT sum("Amount") FROM volume)
    END factor
    ,CASE WHEN (SELECT sum("Amount") FROM volume) = 0 THEN
        CASE WHEN ((SELECT pincr::numeric FROM target) - (SELECT sum("Amount") FROM volume)) <> 0 THEN
            --if the base value is -0- but the target value hasn't been achieved, derive a price to apply
            ((SELECT pincr::numeric FROM target) - (SELECT sum("Amount") FROM volume))/(SELECT sum(app_units) FROM volume)
        ELSE
            0
        END
    ELSE
        0
    END mod_price
FROM
    volume
)
,pricing AS (
SELECT
    o.id
    ,o.logid
    ,o."Trans. Date"
    ,o."Post Date"
    ,o."Description"
    ,(CASE WHEN pscale.factor = 0 THEN o.app_units * pscale.mod_price ELSE o."Amount" * pscale.factor END)::numeric AS Amount
    ,o."Category"
    ,o."Party"
    ,o."Reason"
    ,0::numeric
    ,'app_forecast_name' AS version
    ,'scale price' AS iter
FROM
    volume o
    CROSS JOIN pscale
WHERE
    pscale.factor <> 0 or pscale.mod_price <> 0
)
INSERT INTO
    tpsv.dcard o 
SELECT 
    * 
FROM 
    volume 
UNION ALL 
SELECT 
    * 
FROM 
    pricing

